#!/usr/bin/perl

use lib qw(Net/Intermapper/lib);
use Net::Intermapper;
use Net::Intermapper::Map;

use Data::Dumper;

#my $intermapper = Net::Intermapper->new(hostname=>"10.0.0.1", username=>"admin", password=>"nmsadmin");
#my $map = Net::Intermapper::Map->new(MapName=>"TestMap");
# print Dumper $intermapper->create($map);
# Not supported?!?!?
